@@ -11,9 +11,9 @@
 //#define IPP2P_DEBUG_SOUL
 //#define IPP2P_DEBUG_WINMX
 
-#define get_u8(X,  O)  (*(__u8 *)(X + O))
-#define get_u16(X, O)  get_unaligned((__u16 *)(X + O))
-#define get_u32(X, O)  get_unaligned((__u32 *)(X + O))
+#define get_u8(X,  O)  (*(const __u8 *)((X) + O))
+#define get_u16(X, O)  get_unaligned((const __u16 *)((X) + O))
+#define get_u32(X, O)  get_unaligned((const __u32 *)((X) + O))
 
 MODULE_AUTHOR("Eicke Friedrich/Klaus Degner <ipp2p@ipp2p.org>");
 MODULE_DESCRIPTION("An extension to iptables to identify P2P traffic.");
@@ -21,10 +21,10 @@ MODULE_LICENSE("GPL");
 
 /* Search for UDP eDonkey/eMule/Kad commands */
 static unsigned int
-udp_search_edk(const unsigned char *haystack, const unsigned int packet_len)
+udp_search_edk(const unsigned char *t, const unsigned int packet_len)
 {
-	const unsigned char *t = haystack;
-	t += 8;
+	if (packet_len < 4)
+		return 0;
 
 	switch (t[0]) {
 	case 0xe3:
@@ -32,20 +32,20 @@ udp_search_edk(const unsigned char *haystack, const unsigned int packet_len)
 		switch (t[1]) {
 		/* client -> server status request */
 		case 0x96:
-			if (packet_len == 14)
+			if (packet_len == 6)
 				return IPP2P_EDK * 100 + 50;
 			break;
 
 		/* server -> client status request */
 		case 0x97:
-			if (packet_len == 42)
+			if (packet_len == 34)
 				return IPP2P_EDK * 100 + 51;
 			break;
 
 		/* server description request */
 		/* e3 2a ff f0 .. | size == 6 */
 		case 0xa2:
-			if (packet_len == 14 &&
+			if (packet_len == 6 &&
 			    get_u16(t, 2) == __constant_htons(0xfff0))
 				return IPP2P_EDK * 100 + 52;
 			break;
@@ -59,12 +59,12 @@ udp_search_edk(const unsigned char *haystack, const unsigned int packet_len)
 		*/
 
 		case 0x9a:
-			if (packet_len == 26)
+			if (packet_len == 18)
 				return IPP2P_EDK * 100 + 54;
 			break;
 
 		case 0x92:
-			if (packet_len == 18)
+			if (packet_len == 10)
 				return IPP2P_EDK * 100 + 55;
 			break;
 		}
@@ -72,63 +72,63 @@ udp_search_edk(const unsigned char *haystack, const unsigned int packet_len)
 
 	case 0xe4:
 		switch (t[1]) {
-		/* e4 20 .. | size == 43 */
+		/* e4 20 .. | size == 35 */
 		case 0x20:
-			if (packet_len == 43 && t[2] != 0x00 && t[34] != 0x00)
+			if (packet_len == 35 && t[2] != 0x00 && t[34] != 0x00)
 				return IPP2P_EDK * 100 + 60;
 			break;
 
-		/* e4 00 .. 00 | size == 35 ? */
+		/* e4 00 .. 00 | size == 27 ? */
 		case 0x00:
-			if (packet_len == 35 && t[26] == 0x00)
+			if (packet_len == 27 && t[26] == 0x00)
 				return IPP2P_EDK * 100 + 61;
 			break;
 
-		/* e4 10 .. 00 | size == 35 ? */
+		/* e4 10 .. 00 | size == 27 ? */
 		case 0x10:
-			if (packet_len == 35 && t[26] == 0x00)
+			if (packet_len == 27 && t[26] == 0x00)
 				return IPP2P_EDK * 100 + 62;
 			break;
 
-		/* e4 18 .. 00 | size == 35 ? */
+		/* e4 18 .. 00 | size == 27 ? */
 		case 0x18:
-			if (packet_len == 35 && t[26] == 0x00)
+			if (packet_len == 27 && t[26] == 0x00)
 				return IPP2P_EDK * 100 + 63;
 			break;
 
-		/* e4 52 .. | size = 44 */
+		/* e4 52 .. | size = 36 */
 		case 0x52:
-			if (packet_len == 44)
+			if (packet_len == 36)
 				return IPP2P_EDK * 100 + 64;
 			break;
 
 		/* e4 58 .. | size == 6 */
 		case 0x58:
-			if (packet_len == 14)
+			if (packet_len == 6)
 				return IPP2P_EDK * 100 + 65;
 			break;
 
 		/* e4 59 .. | size == 2 */
 		case 0x59:
-			if (packet_len == 10)
+			if (packet_len == 2)
 				return IPP2P_EDK * 100 + 66;
 			break;
 
-		/* e4 28 .. | packet_len == 52,77,102,127... */
+		/* e4 28 .. | packet_len == 49,69,94,119... */
 		case 0x28:
-			if ((packet_len - 52) % 25 == 0)
+			if ((packet_len - 44) % 25 == 0)
 				return IPP2P_EDK * 100 + 67;
 			break;
 
 		/* e4 50 xx xx | size == 4 */
 		case 0x50:
-			if (packet_len == 12)
+			if (packet_len == 4)
 				return IPP2P_EDK * 100 + 68;
 			break;
 
 		/* e4 40 xx xx | size == 48 */
 		case 0x40:
-			if (packet_len == 56)
+			if (packet_len == 48)
 				return IPP2P_EDK * 100 + 69;
 			break;
 		}
@@ -139,44 +139,36 @@ udp_search_edk(const unsigned char *haystack, const unsigned int packet_len)
 
 /* Search for UDP Gnutella commands */
 static unsigned int
-udp_search_gnu(const unsigned char *haystack, const unsigned int packet_len)
+udp_search_gnu(const unsigned char *t, const unsigned int packet_len)
 {
-	const unsigned char *t = haystack;
-	t += 8;
-
-	if (memcmp(t, "GND", 3) == 0)
+	if (packet_len >= 3 && memcmp(t, "GND", 3) == 0)
 		return IPP2P_GNU * 100 + 51;
-	if (memcmp(t, "GNUTELLA ", 9) == 0)
+	if (packet_len >= 9 && memcmp(t, "GNUTELLA ", 9) == 0)
 		return IPP2P_GNU * 100 + 52;
 	return 0;
 }
 
 /* Search for UDP KaZaA commands */
 static unsigned int
-udp_search_kazaa(const unsigned char *haystack, const unsigned int packet_len)
+udp_search_kazaa(const unsigned char *t, const unsigned int packet_len)
 {
-	const unsigned char *t = haystack;
-
-	if (t[packet_len-1] == 0x00) {
-		t += packet_len - 6;
-		if (memcmp(t, "KaZaA", 5) == 0)
-			return IPP2P_KAZAA * 100 + 50;
-	}
-
+	if (packet_len < 6)
+		return 0;
+	if (memcmp(t + packet_len - 6, "KaZaA\x00", 6) == 0)
+		return IPP2P_KAZAA * 100 + 50;
 	return 0;
 }
 
 /* Search for UDP DirectConnect commands */
-static unsigned int udp_search_directconnect(const unsigned char *haystack,
+static unsigned int udp_search_directconnect(const unsigned char *t,
                                              const unsigned int packet_len)
 {
-	const unsigned char *t = haystack;
-
-	if (*(t + 8) == 0x24 && *(t + packet_len - 1) == 0x7c) {
-		t += 8;
-		if (memcmp(t, "SR ", 3) == 0)
+	if (packet_len < 5)
+		return 0;
+	if (t[0] == 0x24 && t[packet_len-1] == 0x7c) {
+		if (memcmp(&t[1], "SR ", 3) == 0)
 			return IPP2P_DC * 100 + 60;
-		if (memcmp(t, "Ping ", 5) == 0)
+		if (packet_len >= 7 && memcmp(&t[1], "Ping ", 5) == 0)
 			return IPP2P_DC * 100 + 61;
 	}
 	return 0;
@@ -187,77 +179,78 @@ static unsigned int
 udp_search_bit(const unsigned char *haystack, const unsigned int packet_len)
 {
 	switch (packet_len) {
-	case 24:
+	case 16:
 		/* ^ 00 00 04 17 27 10 19 80 */
-		if (ntohl(get_u32(haystack, 8)) == 0x00000417 &&
-		    ntohl(get_u32(haystack, 12)) == 0x27101980)
+		if (ntohl(get_u32(haystack, 0)) == 0x00000417 &&
+		    ntohl(get_u32(haystack, 4)) == 0x27101980)
 			return IPP2P_BIT * 100 + 50;
 		break;
-	case 44:
-		if (get_u32(haystack, 16) == __constant_htonl(0x00000400) &&
-		    get_u32(haystack, 36) == __constant_htonl(0x00000104))
+	case 36:
+		if (get_u32(haystack, 8) == __constant_htonl(0x00000400) &&
+		    get_u32(haystack, 28) == __constant_htonl(0x00000104))
 			return IPP2P_BIT * 100 + 51;
-		if (get_u32(haystack, 16) == __constant_htonl(0x00000400))
+		if (get_u32(haystack, 8) == __constant_htonl(0x00000400))
 			return IPP2P_BIT * 100 + 61;
 		break;
-	case 65:
-		if (get_u32(haystack, 16) == __constant_htonl(0x00000404) &&
-		    get_u32(haystack, 36) == __constant_htonl(0x00000104))
+	case 57:
+		if (get_u32(haystack, 8) == __constant_htonl(0x00000404) &&
+		    get_u32(haystack, 28) == __constant_htonl(0x00000104))
 			return IPP2P_BIT * 100 + 52;
-		if (get_u32(haystack, 16) == __constant_htonl(0x00000404))
+		if (get_u32(haystack, 8) == __constant_htonl(0x00000404))
 			return IPP2P_BIT * 100 + 62;
 		break;
-	case 67:
-		if (get_u32(haystack, 16) == __constant_htonl(0x00000406) && get_u32(haystack, 36) == __constant_htonl(0x00000104))
+	case 59:
+		if (get_u32(haystack, 8) == __constant_htonl(0x00000406) &&
+		    get_u32(haystack, 28) == __constant_htonl(0x00000104))
 			return (IPP2P_BIT * 100 + 53);
-		if (get_u32(haystack, 16) == __constant_htonl(0x00000406))
+		if (get_u32(haystack, 8) == __constant_htonl(0x00000406))
 			return (IPP2P_BIT * 100 + 63);
 		break;
-	case 211:
-		if (get_u32(haystack, 8) == __constant_htonl(0x00000405))
+	case 203:
+		if (get_u32(haystack, 0) == __constant_htonl(0x00000405))
 			return IPP2P_BIT * 100 + 54;
 		break;
-	case 29:
-		if (get_u32(haystack, 8) == __constant_htonl(0x00000401))
+	case 21:
+		if (get_u32(haystack, 0) == __constant_htonl(0x00000401))
 			return IPP2P_BIT * 100 + 55;
 		break;
-	case 52:
-		if (get_u32(haystack, 8)  == __constant_htonl(0x00000827) &&
-		    get_u32(haystack, 12) == __constant_htonl(0x37502950))
+	case 44:
+		if (get_u32(haystack, 0)  == __constant_htonl(0x00000827) &&
+		    get_u32(haystack, 4) == __constant_htonl(0x37502950))
 			return IPP2P_BIT * 100 + 80;
 		break;
 	default:
 		/* this packet does not have a constant size */
-		if (packet_len >= 40 &&
-		    get_u32(haystack, 16) == __constant_htonl(0x00000402) &&
-		    get_u32(haystack, 36) == __constant_htonl(0x00000104))
+		if (packet_len >= 32 &&
+		    get_u32(haystack, 8) == __constant_htonl(0x00000402) &&
+		    get_u32(haystack, 28) == __constant_htonl(0x00000104))
 			return IPP2P_BIT * 100 + 56;
 		break;
 	}
 
 	/* some extra-bitcomet rules: "d1:" [a|r] "d2:id20:" */
-	if (packet_len > 30 && get_u8(haystack, 8) == 'd' &&
-	    get_u8(haystack, 9) == '1' && get_u8(haystack, 10) == ':')
-		if (get_u8(haystack, 11) == 'a' ||
-		    get_u8(haystack, 11) == 'r')
-			if (memcmp(haystack + 12, "d2:id20:", 8) == 0)
+	if (packet_len > 22 && get_u8(haystack, 0) == 'd' &&
+	    get_u8(haystack, 1) == '1' && get_u8(haystack, 2) == ':')
+		if (get_u8(haystack, 3) == 'a' ||
+		    get_u8(haystack, 3) == 'r')
+			if (memcmp(haystack + 4, "d2:id20:", 8) == 0)
 				return IPP2P_BIT * 100 + 57;
 
 #if 0
 	/* bitlord rules */
-	/* packetlen must be bigger than 40 */
+	/* packetlen must be bigger than 32 */
 	/* first 4 bytes are zero */
-	if (packet_len > 40 && get_u32(haystack, 8) == 0x00000000) {
+	if (packet_len > 32 && get_u32(haystack, 0) == 0x00000000) {
 		/* first rule: 00 00 00 00 01 00 00 xx xx xx xx 00 00 00 00*/
-		if (get_u32(haystack, 12) == 0x00000000 &&
-		    get_u32(haystack, 16) == 0x00010000 &&
-		    get_u32(haystack, 24) == 0x00000000)
+		if (get_u32(haystack, 4) == 0x00000000 &&
+		    get_u32(haystack, 8) == 0x00010000 &&
+		    get_u32(haystack, 16) == 0x00000000)
 			return IPP2P_BIT * 100 + 71;
 
 		/* 00 01 00 00 0d 00 00 xx xx xx xx 00 00 00 00*/
-		if (get_u32(haystack, 12) == 0x00000001 &&
-		    get_u32(haystack, 16) == 0x000d0000 &&
-		    get_u32(haystack, 24) == 0x00000000)
+		if (get_u32(haystack, 4) == 0x00000001 &&
+		    get_u32(haystack, 8) == 0x000d0000 &&
+		    get_u32(haystack, 16) == 0x00000000)
 			return IPP2P_BIT * 100 + 71;
 	}
 #endif
@@ -269,6 +262,8 @@ udp_search_bit(const unsigned char *haystack, const unsigned int packet_len)
 static unsigned int
 search_ares(const unsigned char *payload, const unsigned int plen)
 {
+	if (plen < 3)
+		return 0;
 	/* all ares packets start with  */
 	if (payload[1] == 0 && plen - payload[0] == 3) {
 		switch (payload[2]) {
@@ -319,6 +314,8 @@ search_ares(const unsigned char *payload, const unsigned int plen)
 static unsigned int
 search_soul(const unsigned char *payload, const unsigned int plen)
 {
+	if (plen < 8)
+		return 0;
 	/* match: xx xx xx xx | xx = sizeof(payload) - 4 */
 	if (get_u32(payload, 0) == plen - 4) {
 		const uint32_t m = get_u32(payload, 4);
@@ -525,7 +522,7 @@ search_bittorrent(const unsigned char *payload, const unsigned int plen)
 	} else {
 	    	/* bitcomet encryptes the first packet, so we have to detect another
 	    	 * one later in the flow */
-	    	/* first try failed, too many missdetections */
+		/* first try failed, too many false positives */
 	    	/*
 		if (size == 5 && get_u32(t, 0) == __constant_htonl(1) &&
 		    t[4] < 3)
@@ -547,6 +544,8 @@ search_bittorrent(const unsigned char *payload, const unsigned int plen)
 static unsigned int
 search_kazaa(const unsigned char *payload, const unsigned int plen)
 {
+	if (plen < 13)
+		return 0;
 	if (payload[plen-2] == 0x0d && payload[plen-1] == 0x0a &&
 	    memcmp(payload, "GET /.hash=", 11) == 0)
 		return IPP2P_DATA_KAZAA * 100;
@@ -558,10 +557,12 @@ search_kazaa(const unsigned char *payload, const unsigned int plen)
 static unsigned int
 search_gnu(const unsigned char *payload, const unsigned int plen)
 {
+	if (plen < 11)
+		return 0;
 	if (payload[plen-2] == 0x0d && payload[plen-1] == 0x0a) {
 		if (memcmp(payload, "GET /get/", 9) == 0)
 			return IPP2P_DATA_GNU * 100 + 1;
-		if (memcmp(payload, "GET /uri-res/", 13) == 0)
+		if (plen >= 15 && memcmp(payload, "GET /uri-res/", 13) == 0)
 			return IPP2P_DATA_GNU * 100 + 2;
 	}
 	return 0;
@@ -571,53 +572,69 @@ search_gnu(const unsigned char *payload, const unsigned int plen)
 static unsigned int
 search_all_gnu(const unsigned char *payload, const unsigned int plen)
 {
+	if (plen < 11)
+		return 0;
 	if (payload[plen-2] == 0x0d && payload[plen-1] == 0x0a) {
-		if (memcmp(payload, "GNUTELLA CONNECT/", 17) == 0)
+		if (plen >= 19 && memcmp(payload, "GNUTELLA CONNECT/", 17) == 0)
 			return IPP2P_GNU * 100 + 1;
 		if (memcmp(payload, "GNUTELLA/", 9) == 0)
 			return IPP2P_GNU * 100 + 2;
 
-		if (memcmp(payload, "GET /get/", 9) == 0 ||
-		    memcmp(payload, "GET /uri-res/", 13) == 0)
+		if (plen >= 22 && (memcmp(payload, "GET /get/", 9) == 0 ||
+		    memcmp(payload, "GET /uri-res/", 13) == 0))
 		{
-			uint16_t c = 8;
-			const uint16_t end = plen - 22;
+			unsigned int c;
 
-			while (c < end) {
-				if (payload[c] == 0x0a &&
-				    payload[c+1] == 0x0d &&
+			for (c = 0; c < plen - 22; ++c)
+				if (payload[c] == 0x0d &&
+				    payload[c+1] == 0x0a &&
 				    (memcmp(&payload[c+2], "X-Gnutella-", 11) == 0 ||
 				    memcmp(&payload[c+2], "X-Queue:", 8) == 0))
 					return IPP2P_GNU * 100 + 3;
-				c++;
-			}
 		}
 	}
 	return 0;
 }
 
 /* check for KaZaA download commands and other typical data */
+/* plen is guaranteed to be >= 5 (see @matchlist) */
 static unsigned int
 search_all_kazaa(const unsigned char *payload, const unsigned int plen)
 {
-	if (payload[plen-2] == 0x0d && payload[plen-1] == 0x0a) {
-		if (memcmp(payload, "GIVE ", 5) == 0)
-			return IPP2P_KAZAA * 100 + 1;
+	uint16_t c, end, rem;
 
-		if (memcmp(payload, "GET /", 5) == 0) {
-			uint16_t c = 8;
-			const uint16_t end = plen - 22;
+	if (plen < 7)
+		/* too short for anything we test for - early bailout */
+		return 0;
 
-			while (c < end) {
-				if (payload[c] == 0x0a &&
-				    payload[c+1] == 0x0d &&
-				    (memcmp(&payload[c+2], "X-Kazaa-Username: ", 18) == 0 ||
-				    memcmp(&payload[c+2], "User-Agent: PeerEnabler/", 24) == 0))
-					return IPP2P_KAZAA * 100 + 2;
-				c++;
-			}
-		}
+	if (payload[plen-2] != 0x0d || payload[plen-1] != 0x0a)
+		return 0;
+
+	if (memcmp(payload, "GIVE ", 5) == 0)
+		return IPP2P_KAZAA * 100 + 1;
+
+	if (memcmp(payload, "GET /", 5) != 0)
+		return 0;
+
+	if (plen < 18)
+		/* The next tests would not succeed anyhow. */
+		return 0;
+
+	end = plen - 18;
+	rem = plen - 5;
+	for (c = 5; c < end; ++c, --rem) {
+		if (payload[c] != 0x0d)
+			continue;
+		if (payload[c+1] != 0x0a)
+			continue;
+		if (rem >= 18 &&
+		    memcmp(&payload[c+2], "X-Kazaa-Username: ", 18) == 0)
+			return IPP2P_KAZAA * 100 + 2;
+		if (rem >= 24 &&
+		    memcmp(&payload[c+2], "User-Agent: PeerEnabler/", 24) == 0)
+			return IPP2P_KAZAA * 100 + 2;
 	}
+
 	return 0;
 }
 
@@ -625,6 +642,8 @@ search_all_kazaa(const unsigned char *payload, const unsigned int plen)
 static unsigned int
 search_edk(const unsigned char *payload, const unsigned int plen)
 {
+	if (plen < 6)
+		return 0;
 	if (payload[0] != 0xe3) {
 		return 0;
 	} else {
@@ -639,11 +658,12 @@ search_edk(const unsigned char *payload, const unsigned int plen)
 static unsigned int
 search_all_edk(const unsigned char *payload, const unsigned int plen)
 {
+	if (plen < 6)
+		return 0;
 	if (payload[0] != 0xe3) {
 		return 0;
 	} else {
-		//t += head_len;
-		const uint16_t cmd = get_u16(payload, 1);
+		unsigned int cmd = get_u16(payload, 1);
 
 		if (cmd == plen - 5) {
 			switch (payload[5]) {
@@ -663,6 +683,8 @@ search_all_edk(const unsigned char *payload, const unsigned int plen)
 static unsigned int
 search_dc(const unsigned char *payload, const unsigned int plen)
 {
+	if (plen < 6)
+		return 0;
 	if (payload[0] != 0x24) {
 		return 0;
 	} else {
@@ -677,6 +699,8 @@ search_dc(const unsigned char *payload, const unsigned int plen)
 static unsigned int
 search_all_dc(const unsigned char *payload, const unsigned int plen)
 {
+	if (plen < 7)
+		return 0;
 	if (payload[0] == 0x24 && payload[plen-1] == 0x7c) {
 		const unsigned char *t = &payload[1];
 
@@ -688,7 +712,7 @@ search_all_dc(const unsigned char *payload, const unsigned int plen)
 		 * Client-Client-Protocol, some are already recognized by
 		 * client-hub (like lock)
 		 */
-		if (memcmp(t, "MyNick ", 7) == 0)
+		if (plen >= 9 && memcmp(t, "MyNick ", 7) == 0)
 			return IPP2P_DC * 100 + 38;
 	}
 	return 0;
@@ -785,11 +809,9 @@ static const struct {
 };
 
 static bool
-ipp2p_mt(const struct sk_buff *skb, const struct net_device *in,
-         const struct net_device *out, const struct xt_match *match,
-         const void *matchinfo, int offset, unsigned int protoff, bool *hotdrop)
+ipp2p_mt(const struct sk_buff *skb, const struct xt_match_param *par)
 {
-	const struct ipt_p2p_info *info = matchinfo;
+	const struct ipt_p2p_info *info = par->matchinfo;
 	const unsigned char  *haystack;
 	const struct iphdr *ip = ip_hdr(skb);
 	bool p2p_result = false;
@@ -797,9 +819,9 @@ ipp2p_mt(const struct sk_buff *skb, const struct net_device *in,
 	unsigned int hlen = ntohs(ip->tot_len) - ip_hdrlen(skb);	/* hlen = packet-data length */
 
 	/* must not be a fragment */
-	if (offset != 0) {
+	if (par->fragoff != 0) {
 		if (info->debug)
-			printk("IPP2P.match: offset found %i \n", offset);
+			printk("IPP2P.match: offset found %d\n", par->fragoff);
 		return 0;
 	}
 
@@ -815,14 +837,20 @@ ipp2p_mt(const struct sk_buff *skb, const struct net_device *in,
 	switch (ip->protocol) {
 	case IPPROTO_TCP:	/* what to do with a TCP packet */
 	{
-		const struct tcphdr *tcph = tcp_hdr(skb);
+		const struct tcphdr *tcph = (const void *)ip + ip_hdrlen(skb);
 
 		if (tcph->fin) return 0;  /* if FIN bit is set bail out */
 		if (tcph->syn) return 0;  /* if SYN bit is set bail out */
 		if (tcph->rst) return 0;  /* if RST bit is set bail out */
 
 		haystack += tcph->doff * 4; /* get TCP-Header-Size */
-		hlen -= tcph->doff * 4;
+		if (tcph->doff * 4 > hlen) {
+			if (info->debug)
+				pr_info("TCP header indicated packet larger than it is\n");
+			hlen = 0;
+		} else {
+			hlen -= tcph->doff * 4;
+		}
 		while (matchlist[i].command) {
 			if ((info->cmd & matchlist[i].command) == matchlist[i].command &&
 			    hlen > matchlist[i].packet_len)
@@ -842,7 +870,16 @@ ipp2p_mt(const struct sk_buff *skb, const struct net_device *in,
 
 	case IPPROTO_UDP:	/* what to do with an UDP packet */
 	{
-		const struct udphdr *udph = udp_hdr(skb);
+		const struct udphdr *udph = (const void *)ip + ip_hdrlen(skb);
+
+		haystack += sizeof(*udph);
+		if (sizeof(*udph) > hlen) {
+			if (info->debug)
+				pr_info("UDP header indicated packet larger than it is\n");
+			hlen = 0;
+		} else {
+			hlen -= sizeof(*udph);
+		}
 
 		while (udp_list[i].command) {
 			if ((info->cmd & udp_list[i].command) == udp_list[i].command &&
@@ -868,8 +905,8 @@ ipp2p_mt(const struct sk_buff *skb, const struct net_device *in,
 
 static struct xt_match ipp2p_mt_reg __read_mostly = {
 	.name       = "ipp2p",
-	.revision   = 0,
-	.family     = AF_INET,
+	.revision   = 1,
+	.family     = NFPROTO_IPV4,
 	.match      = ipp2p_mt,
 	.matchsize  = sizeof(struct ipt_p2p_info),
 	.me         = THIS_MODULE,
